@@ -17,15 +17,17 @@ use fpm_compiler, only: new_compiler, new_archiver, set_cpp_preprocessor_flags
 
 use fpm_sources, only: add_executable_sources, add_sources_from_dir
 use fpm_targets, only: targets_from_sources, build_target_t, build_target_ptr, &
-                        FPM_TARGET_EXECUTABLE, FPM_TARGET_ARCHIVE
+                        FPM_TARGET_EXECUTABLE, get_library_dirs
 use fpm_manifest, only : get_package_data, package_config_t
 use fpm_meta, only : resolve_metapackages
 use fpm_error, only : error_t, fatal_error, fpm_stop
+use fpm_toml, only: name_is_json
 use, intrinsic :: iso_fortran_env, only : stdin => input_unit, &
                                         & stdout => output_unit, &
                                         & stderr => error_unit
 use iso_c_binding, only: c_char, c_ptr, c_int, c_null_char, c_associated, c_f_pointer
-use fpm_environment, only: os_is_unix
+use fpm_environment, only: os_is_unix, get_os_type, OS_WINDOWS, OS_MACOS, get_env, set_env, delete_env
+use fpm_settings, only: fpm_global_settings, get_global_settings
 
 implicit none
 private
@@ -38,12 +40,13 @@ contains
 subroutine build_model(model, settings, package, error)
     type(fpm_model_t), intent(out) :: model
     class(fpm_build_settings), intent(inout) :: settings
-    type(package_config_t), intent(inout) :: package
+    type(package_config_t), intent(inout), target :: package
     type(error_t), allocatable, intent(out) :: error
 
     integer :: i, j
-    type(package_config_t) :: dependency
-    character(len=:), allocatable :: manifest, lib_dir
+    type(package_config_t), target  :: dependency
+    type(package_config_t), pointer :: manifest
+    character(len=:), allocatable :: file_name, lib_dir
     logical :: has_cpp
     logical :: duplicates_found
     type(string_t) :: include_dir
@@ -66,17 +69,18 @@ subroutine build_model(model, settings, package, error)
     end if
 
     call new_compiler_flags(model,settings)
-    model%build_prefix = join_path("build", basename(model%compiler%fc))
-    model%include_tests = settings%build_tests
+    model%build_prefix         = join_path("build", basename(model%compiler%fc))
+    model%include_tests        = settings%build_tests
     model%enforce_module_names = package%build%module_naming
-    model%module_prefix = package%build%module_prefix
+    model%module_prefix        = package%build%module_prefix
 
     ! Resolve meta-dependencies into the package and the model
     call resolve_metapackages(model,package,settings,error)
     if (allocated(error)) return
-
+    
     ! Create dependencies
-    call new_dependency_tree(model%deps, cache=join_path("build", "cache.toml"))
+    call new_dependency_tree(model%deps, cache=join_path("build", "cache.toml"), &
+    & path_to_config=settings%path_to_config)
 
     ! Build and resolve model dependencies
     call model%deps%add(package, error)
@@ -96,67 +100,60 @@ subroutine build_model(model, settings, package, error)
     has_cpp = .false.
     do i = 1, model%deps%ndep
         associate(dep => model%deps%dep(i))
-            manifest = join_path(dep%proj_dir, "fpm.toml")
+            file_name = join_path(dep%proj_dir, "fpm.toml")
 
-            call get_package_data(dependency, manifest, error, apply_defaults=.true.)
-            if (allocated(error)) exit
-
-            model%packages(i)%name = dependency%name
+            ! The main package manifest should not be reloaded, because it may have been 
+            ! affected by model dependencies and metapackages
+            if (i==1) then 
+                manifest => package
+            else
+                
+                call get_package_data(dependency, file_name, error, apply_defaults=.true.)
+                if (allocated(error)) exit                
+                
+                manifest => dependency
+            end if            
+            
+            model%packages(i)%name = manifest%name
             associate(features => model%packages(i)%features)
-                features%implicit_typing = dependency%fortran%implicit_typing
-                features%implicit_external = dependency%fortran%implicit_external
-                features%source_form = dependency%fortran%source_form
+                features%implicit_typing = manifest%fortran%implicit_typing
+                features%implicit_external = manifest%fortran%implicit_external
+                features%source_form = manifest%fortran%source_form
             end associate
             model%packages(i)%version = package%version%s()
 
             !> Add this dependency's manifest macros
-            allocate(model%packages(i)%macros(0))
-
-            if (allocated(dependency%preprocess)) then
-                do j = 1, size(dependency%preprocess)
-                    if (dependency%preprocess(j)%name == "cpp") then
-                        if (.not. has_cpp) has_cpp = .true.
-                        if (allocated(dependency%preprocess(j)%macros)) then
-                            model%packages(i)%macros = [model%packages(i)%macros, dependency%preprocess(j)%macros]
-                        end if
-                    else
-                        write(stderr, '(a)') 'Warning: Preprocessor ' // package%preprocess(i)%name // &
-                            ' is not supported; will ignore it'
-                    end if
+            if (allocated(manifest%preprocess)) then
+                do j = 1, size(manifest%preprocess)
+                    call model%packages(i)%preprocess%add_config(manifest%preprocess(j))
                 end do
             end if
 
             !> Add this dependency's package-level macros
             if (allocated(dep%preprocess)) then
                 do j = 1, size(dep%preprocess)
-                    if (dep%preprocess(j)%name == "cpp") then
-                        if (.not. has_cpp) has_cpp = .true.
-                        if (allocated(dep%preprocess(j)%macros)) then
-                            model%packages(i)%macros = [model%packages(i)%macros, dep%preprocess(j)%macros]
-                        end if
-                    else
-                        write(stderr, '(a)') 'Warning: Preprocessor ' // package%preprocess(i)%name // &
-                            ' is not supported; will ignore it'
-                    end if
+                    call model%packages(i)%preprocess%add_config(dep%preprocess(j))
                 end do
             end if
 
+            if (model%packages(i)%preprocess%is_cpp()) has_cpp = .true.
+
             if (.not.allocated(model%packages(i)%sources)) allocate(model%packages(i)%sources(0))
 
-            if (allocated(dependency%library)) then
+            if (allocated(manifest%library)) then
 
-                if (allocated(dependency%library%source_dir)) then
-                    lib_dir = join_path(dep%proj_dir, dependency%library%source_dir)
+                if (allocated(manifest%library%source_dir)) then
+                    lib_dir = join_path(dep%proj_dir, manifest%library%source_dir)
                     if (is_dir(lib_dir)) then
                         call add_sources_from_dir(model%packages(i)%sources, lib_dir, FPM_SCOPE_LIB, &
-                            error=error)
+                            with_f_ext=model%packages(i)%preprocess%suffixes, error=error)
                         if (allocated(error)) exit
                     end if
                 end if
 
-                if (allocated(dependency%library%include_dir)) then
-                    do j=1,size(dependency%library%include_dir)
-                        include_dir%s = join_path(dep%proj_dir, dependency%library%include_dir(j)%s)
+                if (allocated(manifest%library%include_dir)) then
+                    do j=1,size(manifest%library%include_dir)
+                        include_dir%s = join_path(dep%proj_dir, manifest%library%include_dir(j)%s)
                         if (is_dir(include_dir%s)) then
                             model%include_dirs = [model%include_dirs, include_dir]
                         end if
@@ -165,17 +162,17 @@ subroutine build_model(model, settings, package, error)
 
             end if
 
-            if (allocated(dependency%build%link)) then
-                model%link_libraries = [model%link_libraries, dependency%build%link]
+            if (allocated(manifest%build%link)) then
+                model%link_libraries = [model%link_libraries, manifest%build%link]
             end if
 
-            if (allocated(dependency%build%external_modules)) then
-                model%external_modules = [model%external_modules, dependency%build%external_modules]
+            if (allocated(manifest%build%external_modules)) then
+                model%external_modules = [model%external_modules, manifest%build%external_modules]
             end if
 
             ! Copy naming conventions from this dependency's manifest
-            model%packages(i)%enforce_module_names = dependency%build%module_naming
-            model%packages(i)%module_prefix        = dependency%build%module_prefix
+            model%packages(i)%enforce_module_names = manifest%build%module_naming
+            model%packages(i)%module_prefix        = manifest%build%module_prefix
 
         end associate
     end do
@@ -187,7 +184,8 @@ subroutine build_model(model, settings, package, error)
     ! Add sources from executable directories
     if (is_dir('app') .and. package%build%auto_executables) then
         call add_sources_from_dir(model%packages(1)%sources,'app', FPM_SCOPE_APP, &
-                                   with_executables=.true., error=error)
+                                   with_executables=.true., with_f_ext=model%packages(1)%preprocess%suffixes,&
+                                   error=error)
 
         if (allocated(error)) then
             return
@@ -196,7 +194,8 @@ subroutine build_model(model, settings, package, error)
     end if
     if (is_dir('example') .and. package%build%auto_examples) then
         call add_sources_from_dir(model%packages(1)%sources,'example', FPM_SCOPE_EXAMPLE, &
-                                   with_executables=.true., error=error)
+                                  with_executables=.true., &
+                                  with_f_ext=model%packages(1)%preprocess%suffixes,error=error)
 
         if (allocated(error)) then
             return
@@ -205,7 +204,8 @@ subroutine build_model(model, settings, package, error)
     end if
     if (is_dir('test') .and. package%build%auto_tests) then
         call add_sources_from_dir(model%packages(1)%sources,'test', FPM_SCOPE_TEST, &
-                                   with_executables=.true., error=error)
+                                  with_executables=.true., &
+                                  with_f_ext=model%packages(1)%preprocess%suffixes,error=error)
 
         if (allocated(error)) then
             return
@@ -215,6 +215,7 @@ subroutine build_model(model, settings, package, error)
     if (allocated(package%executable)) then
         call add_executable_sources(model%packages(1)%sources, package%executable, FPM_SCOPE_APP, &
                                      auto_discover=package%build%auto_executables, &
+                                     with_f_ext=model%packages(1)%preprocess%suffixes, &
                                      error=error)
 
         if (allocated(error)) then
@@ -225,6 +226,7 @@ subroutine build_model(model, settings, package, error)
     if (allocated(package%example)) then
         call add_executable_sources(model%packages(1)%sources, package%example, FPM_SCOPE_EXAMPLE, &
                                      auto_discover=package%build%auto_examples, &
+                                     with_f_ext=model%packages(1)%preprocess%suffixes, &
                                      error=error)
 
         if (allocated(error)) then
@@ -235,6 +237,7 @@ subroutine build_model(model, settings, package, error)
     if (allocated(package%test)) then
         call add_executable_sources(model%packages(1)%sources, package%test, FPM_SCOPE_TEST, &
                                      auto_discover=package%build%auto_tests, &
+                                     with_f_ext=model%packages(1)%preprocess%suffixes, &
                                      error=error)
 
         if (allocated(error)) then
@@ -452,19 +455,26 @@ if (allocated(error)) then
     call fpm_stop(1,'*cmd_build* Model error: '//error%message)
 end if
 
-call targets_from_sources(targets, model, settings%prune, error)
+call targets_from_sources(targets, model, settings%prune, package%library, error)
 if (allocated(error)) then
     call fpm_stop(1,'*cmd_build* Target error: '//error%message)
 end if
 
+!> Dump model to file
+if (len_trim(settings%dump)>0) then
+    call model%dump(trim(settings%dump),error,json=name_is_json(trim(settings%dump)))
+    if (allocated(error)) call fpm_stop(1,'*cmd_build* Model dump error: '//error%message)
+endif
+
 if(settings%list)then
     do i=1,size(targets)
         write(stderr,*) targets(i)%ptr%output_file
     enddo
-else if (settings%show_model) then
+endif
+if (settings%show_model) then
     call show_model(model)
 else
-    call build_package(targets,model,verbose=settings%verbose)
+    call build_package(targets,model,verbose=settings%verbose,dry_run=settings%list)
 endif
 
 end subroutine cmd_build
@@ -484,9 +494,8 @@ subroutine cmd_run(settings,test)
     type(build_target_t), pointer :: exe_target
     type(srcfile_t), pointer :: exe_source
     integer :: run_scope,firsterror
-    integer, allocatable :: stat(:)
-    character(len=:),allocatable :: line
-    logical :: toomany
+    integer, allocatable :: stat(:),target_ID(:)
+    character(len=:),allocatable :: line,run_cmd,library_path
 
     call get_package_data(package, "fpm.toml", error, apply_defaults=.true.)
     if (allocated(error)) then
@@ -498,7 +507,7 @@ subroutine cmd_run(settings,test)
         call fpm_stop(1, '*cmd_run* Model error: '//error%message)
     end if
 
-    call targets_from_sources(targets, model, settings%prune, error)
+    call targets_from_sources(targets, model, settings%prune, package%library, error)
     if (allocated(error)) then
         call fpm_stop(1, '*cmd_run* Targets error: '//error%message)
     end if
@@ -512,47 +521,31 @@ subroutine cmd_run(settings,test)
     ! Enumerate executable targets to run
     col_width = -1
     found(:) = .false.
-    allocate(executables(0))
-    do i=1,size(targets)
-
+    allocate(executables(size(targets)),target_ID(size(targets)))
+    enumerate: do i=1,size(targets)
         exe_target => targets(i)%ptr
-
-        if (exe_target%target_type == FPM_TARGET_EXECUTABLE .and. &
-             allocated(exe_target%dependencies)) then
-
+        if (should_be_run(settings,run_scope,exe_target)) then  
+            
             exe_source => exe_target%dependencies(1)%ptr%source
-
-            if (exe_source%unit_scope == run_scope) then
-
-                col_width = max(col_width,len(basename(exe_target%output_file))+2)
-
-                if (size(settings%name) == 0) then
-
-                    exe_cmd%s = exe_target%output_file
-                    executables = [executables, exe_cmd]
-
-                else
-
-                    do j=1,size(settings%name)
-
-                        if (glob(trim(exe_source%exe_name),trim(settings%name(j)))) then
-
-                            found(j) = .true.
-                            exe_cmd%s = exe_target%output_file
-                            executables = [executables, exe_cmd]
-
-                        end if
-
-                    end do
-
-                end if
-
-            end if
-
-        end if
-
-    end do
-
+                
+            col_width = max(col_width,len(basename(exe_target%output_file))+2)
+            
+            ! Priority by name ID, or 0 if no name present (run first)
+            j              = settings%name_ID(exe_source%exe_name)
+            target_ID(i)   = j
+            if (j>0) found(j) = .true.
+            
+            exe_cmd%s      = exe_target%output_file
+            executables(i) = exe_cmd
+            
+        else
+            target_ID(i)   = huge(target_ID(i))
+        endif
+    end do enumerate
+    
+    ! sort executables by ascending name ID, resize
+    call sort_executables(target_ID,executables)
+    
     ! Check if any apps/tests were found
     if (col_width < 0) then
         if (test) then
@@ -562,16 +555,9 @@ subroutine cmd_run(settings,test)
         end if
     end if
 
-
-
     ! Check all names are valid
-    ! or no name and found more than one file
-    toomany= size(settings%name)==0 .and. size(executables)>1
-    if ( any(.not.found) &
-    & .or. &
-    & ( (toomany .and. .not.test) .or.  (toomany .and. settings%runner /= '') ) &
-    & .and. &
-    & .not.settings%list) then
+    ! or no name and found more than one file    
+    if ( any(.not.found) ) then
         line=join(settings%name)
         if(line/='.')then ! do not report these special strings
            if(any(.not.found))then
@@ -597,31 +583,29 @@ subroutine cmd_run(settings,test)
 
     end if
 
-    call build_package(targets,model,verbose=settings%verbose)
+    call build_package(targets,model,verbose=settings%verbose,dry_run=settings%list)
 
     if (settings%list) then
          call compact_list()
     else
 
+        ! Save current library path and set a new one that includes the local 
+        ! dynamic library folders
+        library_path = save_library_path()                
+        call set_library_path(model, targets, error)
+        if (allocated(error)) call fpm_stop(1, '*cmd_run* Run error: '//error%message)
+
         allocate(stat(size(executables)))
         do i=1,size(executables)
             if (exists(executables(i)%s)) then
-                if(settings%runner /= ' ')then
-                    if(.not.allocated(settings%args))then
-                       call run(settings%runner_command()//' '//executables(i)%s, &
-                             echo=settings%verbose, exitstat=stat(i))
-                    else
-                       call run(settings%runner_command()//' '//executables(i)%s//" "//settings%args, &
-                             echo=settings%verbose, exitstat=stat(i))
-                    endif
-                else
-                    if(.not.allocated(settings%args))then
-                       call run(executables(i)%s,echo=settings%verbose, exitstat=stat(i))
-                    else
-                       call run(executables(i)%s//" "//settings%args,echo=settings%verbose, &
-                             exitstat=stat(i))
-                    endif
-                endif
+                
+                ! Prepare command line
+                                              run_cmd = executables(i)%s
+                if (settings%runner/=' ')     run_cmd = settings%runner_command()//' '//run_cmd
+                if (allocated(settings%args)) run_cmd = run_cmd//" "//settings%args
+                
+                call run(run_cmd,echo=settings%verbose,exitstat=stat(i))                
+                
             else
                 call fpm_stop(1,'*cmd_run*:'//executables(i)%s//' not found')
             end if
@@ -637,6 +621,10 @@ subroutine cmd_run(settings,test)
             firsterror = findloc(stat/=0,value=.true.,dim=1)
             call fpm_stop(stat(firsterror),'*cmd_run*:stopping due to failed executions')
         end if
+        
+        ! Restore original library path
+        call restore_library_path(library_path, error)
+        if (allocated(error)) call fpm_stop(1, '*cmd_run* Environment error: '//error%message)                         
 
     end if
 
@@ -698,21 +686,30 @@ subroutine delete_skip(is_unix)
     end do
 end subroutine delete_skip
 
-!> Delete the build directory including or excluding dependencies.
+!> Delete the build directory including or excluding dependencies. Can be used
+!> to clear the registry cache.
 subroutine cmd_clean(settings)
     !> Settings for the clean command.
     class(fpm_clean_settings), intent(in) :: settings
 
     character :: user_response
+    type(fpm_global_settings) :: global_settings
+    type(error_t), allocatable :: error
+
+    ! Clear registry cache
+    if (settings%registry_cache) then
+        call get_global_settings(global_settings, error) 
+        if (allocated(error)) return
+
+        call os_delete_dir(os_is_unix(), global_settings%registry_settings%cache_path)
+    end if
 
     if (is_dir('build')) then
         ! Remove the entire build directory
-        if (settings%clean_call) then
+        if (settings%clean_all) then
             call os_delete_dir(os_is_unix(), 'build'); return
-        end if
-
         ! Remove the build directory but skip dependencies
-        if (settings%clean_skip) then
+        else if (settings%clean_skip) then
             call delete_skip(os_is_unix()); return
         end if
 
@@ -725,4 +722,165 @@ subroutine cmd_clean(settings)
     end if
 end subroutine cmd_clean
 
+!> Sort executables by namelist ID, and trim unused values
+pure subroutine sort_executables(target_ID,executables)
+    integer, allocatable, intent(inout) :: target_ID(:)
+    type(string_t), allocatable, intent(inout) :: executables(:)
+    
+    integer :: i,j,n,used
+    
+    n = size(target_ID)
+    used = 0
+    
+    sort: do i=1,n
+       do j=i+1,n
+          if (target_ID(j)<target_ID(i)) &
+          call swap(target_ID(i),target_ID(j),executables(i),executables(j))
+       end do
+       if (target_ID(i)<huge(target_ID(i))) used = i
+    end do sort   
+    
+    if (used>0 .and. used<n) then 
+        target_ID = target_ID(1:used)
+        executables = executables(1:used)
+    end if
+    
+    contains
+    
+    elemental subroutine swap(t1,t2,e1,e2)
+       integer, intent(inout) :: t1,t2
+       type(string_t), intent(inout) :: e1,e2
+       integer :: tmp
+       type(string_t) :: etmp
+       
+       tmp = t1
+       t1  = t2
+       t2  = tmp 
+       etmp = e1
+       e1  = e2
+       e2  = etmp        
+    end subroutine swap
+    
+end subroutine sort_executables
+
+!> Check if an executable should be run 
+logical function should_be_run(settings,run_scope,exe_target)
+    class(fpm_run_settings), intent(in) :: settings
+    integer, intent(in) :: run_scope
+    type(build_target_t), intent(in) :: exe_target
+    
+    if (exe_target%is_executable_target(run_scope)) then
+        
+        associate(exe_source => exe_target%dependencies(1)%ptr%source)
+            
+            if (exe_source%unit_scope/=run_scope) then 
+                
+                ! Other scope
+                should_be_run = .false.
+                
+            elseif (size(settings%name) == 0 .or. settings%list) then
+
+                ! Run all or list all
+                should_be_run = .true.
+
+            else
+
+                ! Is found in list
+                should_be_run = settings%name_ID(exe_source%exe_name)>0
+                
+            end if            
+            
+        end associate
+                        
+    else
+        
+        !> Invalid target
+        should_be_run = .false.
+        
+    endif
+    
+end function should_be_run
+
+!> Save the current runtime library path (e.g., PATH or LD_LIBRARY_PATH)
+function save_library_path() result(path)
+    character(len=:), allocatable :: path
+
+    select case (get_os_type())
+    case (OS_WINDOWS)
+        path = get_env("PATH", default="")
+    case (OS_MACOS)
+        ! macOS does not use LD_LIBRARY_PATH by default for `.dylib`
+        allocate(character(0) :: path)
+    case default ! UNIX/Linux
+        path = get_env("LD_LIBRARY_PATH", default="")
+    end select
+end function save_library_path
+
+!> Set the runtime library path for the current process (used for subprocesses)
+subroutine set_library_path(model, targets, error)
+    type(fpm_model_t), intent(in) :: model
+    type(build_target_ptr), intent(inout), target :: targets(:)
+    type(error_t), allocatable, intent(out) :: error
+     
+    type(string_t), allocatable :: shared_lib_dirs(:)
+    character(len=:), allocatable :: new_path, sep
+    logical :: success
+    integer :: i
+
+    ! Get library directories
+    call get_library_dirs(model, targets, shared_lib_dirs)
+
+    ! Select platform-specific separator
+    select case (get_os_type())
+    case (OS_WINDOWS)
+        sep = ";"
+    case default
+        sep = ":"
+    end select
+
+    ! Join the directories into a path string
+    ! Manually join paths
+    new_path = ""
+    do i = 1, size(shared_lib_dirs)
+        if (i > 1) new_path = new_path // sep
+        new_path = new_path // shared_lib_dirs(i)%s
+    end do    
+
+    ! Set the appropriate environment variable
+    select case (get_os_type())
+    case (OS_WINDOWS)
+        success = set_env("PATH", new_path // sep // get_env("PATH", default=""))
+    case (OS_MACOS)
+        ! Typically not required for local .dylib use, noop or DYLD_LIBRARY_PATH if needed
+        success = .true.
+    case default ! UNIX/Linux
+        success = set_env("LD_LIBRARY_PATH", new_path // sep // get_env("LD_LIBRARY_PATH", default=""))
+    end select
+    
+    if (.not.success) call fatal_error(error," Cannot set library path: "//new_path)
+
+end subroutine set_library_path
+
+!> Restore a previously saved runtime library path
+subroutine restore_library_path(saved_path,error)
+    character(*), intent(in) :: saved_path
+    type(error_t), allocatable, intent(out) :: error
+    logical :: success
+
+    select case (get_os_type())
+    case (OS_WINDOWS)
+        success = set_env("PATH", saved_path)
+    case (OS_MACOS)
+        ! noop
+        success = .true.
+    case default ! UNIX/Linux
+        success = set_env("LD_LIBRARY_PATH", saved_path)
+    end select
+    
+    if (.not.success) call fatal_error(error, "Cannot restore library path "//saved_path)
+
+end subroutine restore_library_path
+
+
+
 end module fpm
