@@ -6,6 +6,8 @@ module test_manifest
     use fpm_manifest_profile, only: profile_config_t, find_profile
     use fpm_strings, only: operator(.in.), string_t
     use fpm_error, only: fatal_error, error_t
+    use tomlf, only : new_table, toml_table, toml_array
+    use fpm_toml, only : add_table, add_array, get_value, get_list, set_value, set_list
     implicit none
     private
     public :: collect_manifest
@@ -260,7 +262,6 @@ contains
     !> Dependencies cannot be created from empty tables
     subroutine test_dependency_empty(error)
         use fpm_manifest_dependency
-        use fpm_toml, only : new_table, toml_table
 
         !> Error handling
         type(error_t), allocatable, intent(out) :: error
@@ -279,7 +280,6 @@ contains
     !> Try to create a dependency with conflicting entries
     subroutine test_dependency_pathtag(error)
         use fpm_manifest_dependency
-        use fpm_toml, only : new_table, toml_table, set_value
 
         !> Error handling
         type(error_t), allocatable, intent(out) :: error
@@ -301,7 +301,6 @@ contains
     !> Try to create a dependency with conflicting entries
     subroutine test_dependency_nourl(error)
         use fpm_manifest_dependency
-        use fpm_toml, only : new_table, toml_table, set_value
 
         !> Error handling
         type(error_t), allocatable, intent(out) :: error
@@ -322,7 +321,6 @@ contains
     !> Try to create a dependency with conflicting entries
     subroutine test_dependency_gitpath(error)
         use fpm_manifest_dependency
-        use fpm_toml, only : new_table, toml_table, set_value
 
         !> Error handling
         type(error_t), allocatable, intent(out) :: error
@@ -344,7 +342,6 @@ contains
     !> Try to create a dependency with conflicting entries
     subroutine test_dependency_gitconflict(error)
         use fpm_manifest_dependency
-        use fpm_toml, only : new_table, toml_table, set_value
 
         !> Error handling
         type(error_t), allocatable, intent(out) :: error
@@ -367,7 +364,6 @@ contains
     !> Try to create a git dependency with an invalid source format.
     subroutine test_dependency_invalid_git(error)
         use fpm_manifest_dependency
-        use fpm_toml, only : new_table, toml_table, set_value
 
         !> Error handling
         type(error_t), allocatable, intent(out) :: error
@@ -386,7 +382,6 @@ contains
     !> Namespace is necessary if a dependency is not a git or path dependency
     subroutine test_dependency_no_namespace(error)
         use fpm_manifest_dependency
-        use fpm_toml, only : new_table, toml_table, set_value
 
         type(error_t), allocatable, intent(out) :: error
 
@@ -404,7 +399,6 @@ contains
     !> Do not specify version with a git or path dependency
     subroutine test_dependency_redundant_v(error)
         use fpm_manifest_dependency
-        use fpm_toml, only : new_table, toml_table, set_value
 
         type(error_t), allocatable, intent(out) :: error
 
@@ -424,7 +418,6 @@ contains
     !> Try to create a dependency with conflicting entries
     subroutine test_dependency_wrongkey(error)
         use fpm_manifest_dependency
-        use fpm_toml, only : new_table, toml_table, set_value
 
         !> Error handling
         type(error_t), allocatable, intent(out) :: error
@@ -445,7 +438,6 @@ contains
     !> Dependency tables can be empty
     subroutine test_dependencies_empty(error)
         use fpm_manifest_dependency
-        use fpm_toml, only : new_table, toml_table
 
         !> Error handling
         type(error_t), allocatable, intent(out) :: error
@@ -468,7 +460,6 @@ contains
     !> Add a dependency as an array, which is not supported
     subroutine test_dependencies_typeerror(error)
         use fpm_manifest_dependency
-        use fpm_toml, only : new_table, add_array, toml_table, toml_array
 
         !> Error handling
         type(error_t), allocatable, intent(out) :: error
@@ -603,7 +594,6 @@ contains
     !> Executables cannot be created from empty tables
     subroutine test_executable_empty(error)
         use fpm_manifest_executable
-        use fpm_toml, only : new_table, toml_table
 
         !> Error handling
         type(error_t), allocatable, intent(out) :: error
@@ -621,7 +611,6 @@ contains
     !> Pass a wrong TOML type to the name field of the executable
     subroutine test_executable_typeerror(error)
         use fpm_manifest_executable
-        use fpm_toml, only : new_table, add_table, toml_table
 
         !> Error handling
         type(error_t), allocatable, intent(out) :: error
@@ -642,7 +631,6 @@ contains
     !> Pass a TOML table with insufficient entries to the executable constructor
     subroutine test_executable_noname(error)
         use fpm_manifest_executable
-        use fpm_toml, only : new_table, add_table, toml_table
 
         !> Error handling
         type(error_t), allocatable, intent(out) :: error
@@ -663,7 +651,6 @@ contains
     !> Pass a TOML table with not allowed keys
     subroutine test_executable_wrongkey(error)
         use fpm_manifest_executable
-        use fpm_toml, only : new_table, add_table, toml_table
 
         !> Error handling
         type(error_t), allocatable, intent(out) :: error
@@ -838,7 +825,6 @@ contains
     !> Libraries can be created from empty tables
     subroutine test_library_empty(error)
         use fpm_manifest_library
-        use fpm_toml, only : new_table, toml_table
 
         !> Error handling
         type(error_t), allocatable, intent(out) :: error
@@ -871,7 +857,6 @@ contains
     !> Pass a TOML table with not allowed keys
     subroutine test_library_wrongkey(error)
         use fpm_manifest_library
-        use fpm_toml, only : new_table, add_table, toml_table
 
         !> Error handling
         type(error_t), allocatable, intent(out) :: error
@@ -891,7 +876,6 @@ contains
    !> Pass a TOML table with not allowed source dirs
     subroutine test_library_list(error)
         use fpm_manifest_library
-        use fpm_toml, only : new_table, set_list, toml_table
 
         !> Error handling
         type(error_t), allocatable, intent(out) :: error
@@ -910,7 +894,6 @@ contains
     !> Pass a TOML table with a 1-sized source dir list
     subroutine test_library_listone(error)
         use fpm_manifest_library
-        use fpm_toml, only : new_table, set_list, toml_table
 
         !> Error handling
         type(error_t), allocatable, intent(out) :: error
@@ -933,8 +916,6 @@ contains
     !> Packages cannot be created from empty tables
     subroutine test_package_simple(error)
         use fpm_manifest_package
-        use fpm_toml, only : new_table, add_table, add_array, set_value, &
-            & toml_table, toml_array
 
         !> Error handling
         type(error_t), allocatable, intent(out) :: error
@@ -972,7 +953,6 @@ contains
     !> Packages cannot be created from empty tables
     subroutine test_package_empty(error)
         use fpm_manifest_package
-        use fpm_toml, only : new_table, toml_table
 
         !> Error handling
         type(error_t), allocatable, intent(out) :: error
@@ -990,7 +970,6 @@ contains
     !> Create an array in the package name, which should cause an error
     subroutine test_package_typeerror(error)
         use fpm_manifest_package
-        use fpm_toml, only : new_table, add_array, toml_table, toml_array
 
         !> Error handling
         type(error_t), allocatable, intent(out) :: error
@@ -1011,7 +990,6 @@ contains
     !> Try to create a new package without a name field
     subroutine test_package_noname(error)
         use fpm_manifest_package
-        use fpm_toml, only : new_table, add_table, toml_table
 
         !> Error handling
         type(error_t), allocatable, intent(out) :: error
@@ -1034,7 +1012,6 @@ contains
     !> Try to read executables from a mixed type array
     subroutine test_package_wrongexe(error)
         use fpm_manifest_package
-        use fpm_toml, only : new_table, set_value, add_array, toml_table, toml_array
 
         !> Error handling
         type(error_t), allocatable, intent(out) :: error
@@ -1057,7 +1034,6 @@ contains
     !> Try to read tests from a mixed type array
     subroutine test_package_wrongtest(error)
         use fpm_manifest_package
-        use fpm_toml, only : new_table, set_value, add_array, toml_table, toml_array
 
         !> Error handling
         type(error_t), allocatable, intent(out) :: error
@@ -1080,7 +1056,6 @@ contains
     !> Try to read tests from a mixed type array
     subroutine test_package_duplicate(error)
         use fpm_manifest_package
-        use fpm_toml, only : set_value, add_table, add_array, toml_table, toml_array
 
         !> Error handling
         type(error_t), allocatable, intent(out) :: error
@@ -1107,7 +1082,6 @@ contains
     !> Tests cannot be created from empty tables
     subroutine test_test_simple(error)
         use fpm_manifest_test
-        use fpm_toml, only : new_table, set_value, add_table, toml_table
 
         !> Error handling
         type(error_t), allocatable, intent(out) :: error
@@ -1135,7 +1109,6 @@ contains
     !> Tests cannot be created from empty tables
     subroutine test_test_empty(error)
         use fpm_manifest_test
-        use fpm_toml, only : new_table, toml_table
 
         !> Error handling
         type(error_t), allocatable, intent(out) :: error
@@ -1153,7 +1126,6 @@ contains
     !> Pass a wrong TOML type to the name field of the test
     subroutine test_test_typeerror(error)
         use fpm_manifest_test
-        use fpm_toml, only : new_table, add_table, toml_table
 
         !> Error handling
         type(error_t), allocatable, intent(out) :: error
@@ -1174,7 +1146,6 @@ contains
     !> Pass a TOML table with insufficient entries to the test constructor
     subroutine test_test_noname(error)
         use fpm_manifest_test
-        use fpm_toml, only : new_table, add_table, toml_table
 
         !> Error handling
         type(error_t), allocatable, intent(out) :: error
@@ -1195,7 +1166,6 @@ contains
     !> Pass a TOML table with not allowed keys
     subroutine test_test_wrongkey(error)
         use fpm_manifest_test
-        use fpm_toml, only : new_table, add_table, toml_table
 
         !> Error handling
         type(error_t), allocatable, intent(out) :: error
@@ -1216,7 +1186,6 @@ contains
     !> Create a simple example entry
     subroutine test_example_simple(error)
         use fpm_manifest_example
-        use fpm_toml, only : new_table, set_value, add_table, toml_table
 
         !> Error handling
         type(error_t), allocatable, intent(out) :: error
@@ -1244,7 +1213,6 @@ contains
     !> Examples cannot be created from empty tables
     subroutine test_example_empty(error)
         use fpm_manifest_example
-        use fpm_toml, only : new_table, toml_table
 
         !> Error handling
         type(error_t), allocatable, intent(out) :: error
@@ -1262,7 +1230,6 @@ contains
     !> Test link options
     subroutine test_link_string(error)
         use fpm_manifest_build
-        use fpm_toml, only : set_value, toml_table
 
         !> Error handling
         type(error_t), allocatable, intent(out) :: error
@@ -1287,7 +1254,6 @@ contains
     !> Test link options
     subroutine test_link_array(error)
         use fpm_manifest_build
-        use fpm_toml, only : add_array, set_value, toml_table, toml_array
 
         !> Error handling
         type(error_t), allocatable, intent(out) :: error
@@ -1315,7 +1281,6 @@ contains
     !> Test link options
     subroutine test_invalid_link(error)
         use fpm_manifest_build
-        use fpm_toml, only : add_table, toml_table
 
         !> Error handling
         type(error_t), allocatable, intent(out) :: error
@@ -1335,7 +1300,6 @@ contains
 
     subroutine test_install_library(error)
         use fpm_manifest_install
-        use fpm_toml, only : toml_table, set_value
 
         !> Error handling
         type(error_t), allocatable, intent(out) :: error
@@ -1359,7 +1323,6 @@ contains
 
     subroutine test_install_empty(error)
         use fpm_manifest_install
-        use fpm_toml, only : toml_table
 
         !> Error handling
         type(error_t), allocatable, intent(out) :: error
@@ -1382,7 +1345,6 @@ contains
 
     subroutine test_install_wrongkey(error)
         use fpm_manifest_install
-        use fpm_toml, only : toml_table, set_value
 
         !> Error handling
         type(error_t), allocatable, intent(out) :: error
@@ -1399,7 +1361,6 @@ contains
 
     subroutine test_preprocess_empty(error)
         use fpm_manifest_preprocess
-        use fpm_toml, only : new_table, toml_table
 
         !> Error handling
         type(error_t), allocatable, intent(out) :: error
@@ -1410,14 +1371,13 @@ contains
         call new_table(table)
         table%key = "example"
 
-        call new_preprocess_config(preprocess, table, error)
+        call preprocess%new(table, error)
 
     end subroutine test_preprocess_empty
 
     !> Pass a TOML table with not allowed keys
     subroutine test_preprocess_wrongkey(error)
         use fpm_manifest_preprocess
-        use fpm_toml, only : new_table, add_table, toml_table
 
         !> Error handling
         type(error_t), allocatable, intent(out) :: error
@@ -1431,14 +1391,13 @@ contains
         table%key = 'example'
         call add_table(table, 'wrong-field', child, stat)
 
-        call new_preprocess_config(preprocess, table, error)
+        call preprocess%new(table, error)
 
     end subroutine test_preprocess_wrongkey
 
     !> Preprocess table cannot be empty.
     subroutine test_preprocessors_empty(error)
         use fpm_manifest_preprocess
-        use fpm_toml, only : new_table, toml_table
 
         !> Error handling
         type(error_t), allocatable, intent(out) :: error
